/*
* 32bit vector multiplying implementation. Round at 1bit under LSB twice.
*
* function prototype
* void fr32_vector_mul(
*        const fract32 a[],
*        const fract32 b[],
*        fract32 c[],	// a*c
*        int count);
*
* parameters
*   FP+20	-	int count
*   FP+16	R2	const fr32 c[] : as result
*	FP+12	R1	const fr32 b[]
*	FP+ 8	R0	const fr32 a[]
*
* return
*	none
*
* side effect
*   out[] : obtain output data.
*

* register layout
*   P2 : out
*	P3 : count : loop counter's initial value
*   I0 : a[]
*   I1 : b[]
*   I2 : c[]
*   R0 : a[i]
*   R1 : b[i]
*	R6 : a[i]*b[i]
*	R7 : rounding constant.
*/

	.text
	.align 4
	.global _fr32_vector_mul;
	.type _fr32_vector_mul, STT_FUNC;

_fr32_vector_mul:
	link	0;
	[--sp] = (r7:4, p5:3);		// save all preserved register

		/* Set up registers */
	i0 = r0;
	i1 = r1;
	i2 = r2;
	p3 = [fp+20];				// load count
	r7.h = 0x4000;				// bit 14
	r7.l = 0;
	r5 = 0x4000(z);

	a0 = r7;
		/* outer loop */
	loop count lc0 = p3;
	loop_begin count;

				// r7 is bias of rounding point
				/* first half*/
		a1 = r5 || r0 = [i0++] || r1 = [i1++];				// load coeff & delaydata
		a1 += r0.H * r1.L (m), a0 += r0.L * r1.L (fu);

				/* Second half */
		a0 = a0 >> 31;					// scale down the unsigned integer.

		a0 += r1.H * r0.H, a1 += r1.H * r0.L (m);

		a1 = a1 >>> 15;				// scale down the signed integer
		r6 = (a0 += a1);
		a0 = r7 || [i2++] = r6;				// store output sample



	loop_end count;
		/* end of outer loop */


	(r7:4, p5:3) = [sp++];		// restore all preserved register
	unlink;
	rts;
	.size	_fr32_vector_mul, .-_fr32_vector_mul
